#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
 *  Copyright (C) 2007 Karlheinz Klingbeil (lunqual)
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MeRCHANTABILITY or FITNeSS FOR A PARTICULAR PURPOSe.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 
 Das ist eine Referenzimplementierung der amtlichen Alkoholtafeln 2-6
 
 Wichtig: Die Funktionen der Klasse Tafel prüfen grundsätzlich
 keine Eingabewerte !
 Beachten Sie daher, dass Programme, die diese Klasse nutzen, die
 eingegebene Alkoholstärke daraufhin überprüfen, ob diese sinnvoll
 ist, d.h. größer als 0 und kleiner als 100.
 Ferner ist darauf zu achten, dass diese Formel nur für 
 Temperaturen zwischen -30 und +20 Grad C spezifiziert ist.
 Temperaturen ausserhalb dieses Bereichs werden zwar berechnet,
 die Ergebnisse entsprechen aber nicht mehr den amtlichen Alkoholtafeln !
'''
 
#------------------------
# Tafelrechner imports
#------------------------
import getopt    # wird für die Auswertung der Kommandozeile benötigt
import sys       # dito

#------------------------
# Konstanten
#------------------------
VERSION = 'T0.2/18.09.07'


#------------------------
# Die Klasse "Tafel" stellt 
# Rechenfunktionen für die amtlichen Alkoholtafeln
# 2-6 zur Verfügung
#------------------------
class Tafel:
    import math
    #----------------------------
    # Dichte von Wasser und
    # reinem Alkohol
    # nullPrecision gibt die
    # Rechengenauigkeit der
    # Formel vor, größere Werte
    # rechnen ungenauer !
    # Alle Funktionen rechnen in voller Genauigkeit !,
    # d.h. die Rundung der gedruckten amtlichen 
    # Alkoholtafeln erfolgt in diesem Programm
    # erst bei der Ausgabe, nicht schon während 
    # der Berechnung !
    #----------------------------
    rhoWasser       = 0.99820123     # Dichte von Wasser bei 20 °C
    rhoAlk          = 0.789239       # Dichte von reinem Alkohol bei 20 °C
    nullPrecision   = 0.0001         # weiter oben erklärt
    
    #----------------------------
    #
    # Koeffizienten der Alkoholformel
    # nach IPT68
    #
    #----------------------------
    co = [998.20123,-192.9769495,389.1238958,\
          -1668.103923,13522.15441,-88292.78388,\
          306287.4042,-613838.1234,747017.2998, \
          -547846.1354,223446.0334,-39032.85426 ]
        
    cb = [-2.0618513e-1,-5.2682542e-3,3.6130013e-5,\
          -3.8957702e-7,7.169354e-9,-9.9739231e-11 ]
    
    ck = [\
            [ 1.693443461530087e-1,-1.046914743455169e+1,7.196353469546523e+1,\
             -7.047478054272792e+2,3.924090430035045e+3,-1.210164659068747e+4,\
             2.248646550400788e+4,-2.605562982188164e+4,1.852373922069467e+4,\
            -7.420201433430137e+3,1.285617841998974e+3\
            ],[\
             -1.19301300505701e-2,2.517399633803461e-1,-2.170575700563993,\
             1.353034988843029e+1,-5.029988758537014e+1,1.09635566657757e+2,\
             -1.422753946421155e+2,1.08043594285623e+2,-4.414153236817392e+1,\
             7.442971530188783,0
            ],[\
             -6.802995733503803e-4,1.876837790289664e-2,-0.2002561813734156,\
             1.02299296671922,-2.895696483903638,4.810060584300675,\
             -4.672147440794683,2.458043105903461,-5.411227621436812e-1,0,0\
            ],[\
             4.075376675622027e-6,-8.76305857347111e-6,6.515031360099368e-6,\
             -1.51578483698721e-6,0,0,0,0,0,0,0\
            ],[
             -2.788074354782409e-08,1.345612883493354e-08,\
             0,0,0,0,0,0,0,0,0\
            ]\
        ]
    
    ci = [11,10,9,4,2]
 
    #---------------------
    # Tafel 2
    # mit Temperaturangabe
    #
    #---------------------
    def Tafel2(self,VolProz,Temperatur=20.0):
        v=0.0
        vt = self.Tafel6(VolProz, Temperatur) * 1000.0;
        v = self.dt6(self.Tafel4(VolProz)) * 1000.0;
        t2 = ((100.0 * VolProz * (1000.0 / v) * (1 + (1.2 / vt) - 0.00015)) ) / 10.0;
        return float(str('%4.1f' % (t2)))
    
    #---------------------
    # Tafel 3 Hilfsfunktion
    # ermittelt den Wert für eine bestimmte Stärke und 
    # Temperatur
    #
    # Für den korrekt interpolierten Wert ist 
    # die Funktion "Tafel3" zu verwenden !
    #---------------------
    def Tafel3_b(self,VolProz,Temperatur=20.0):
        dichte_20 = 0.0
        dichte_t  = 0.0
        dichte_20 = self.dt6(self.Tafel4(VolProz));
        dichte_t  = self.Tafel6(VolProz, Temperatur);
        return (VolProz / 100.0) * (dichte_t / dichte_20) * (1000.0 * (1 + (0.000036 * (Temperatur - 20.0))));
    
    #-------------------------
    #
    # Tafel 3 mit Temperaturangabe
    # und Interpolierung nach Vorschrift
    #------------------------
    def Tafel3(self,VolProz, Temperatur=20.0 ):
        oben = self.Tafel3_b(self.math.floor(VolProz + 1.0), Temperatur) 
        unten = self.Tafel3_b(self.math.floor(VolProz), Temperatur )
        diff = VolProz - self.math.floor(VolProz)
        return ((oben -unten) *  diff) + unten    
    
    #-------------------
    # Massenprozent aus Volumenprozent (Tafel 4)
    #-------------------
    
    def Tafel4(self, VolProz):
        alt = 0.0
        neu = 0.0
        neu = VolProz * self.rhoAlk / self.dt6(VolProz)
        while abs(alt - neu) > self.nullPrecision:
            alt = neu
            neu = VolProz * self.rhoAlk / self.dt6(alt);
        return neu
    
    #-------------------
    # Volumenprozent aus Massenprozent (Tafel 5)
    #------------------
    def Tafel5(self, MasProz):
        return (MasProz * self.dt6(MasProz)) / self.rhoAlk    
    
    #---------------------
    # Hilfsrechnung für die Dichteermittlung
    # ergibt die Dichte (Tafel 6) bei 20 Grad
    # Grundlage der Berechnungnen
    # Dies ist der erste Teil der Formel, der
    # nur die Dichte ohne Korrektur der Temperatur
    # berechnet.
    # Der zweite Teil (Korrektur der Temperatur
    # wird in der Funktion "Tafel6" angewendet
    #---------------------
    def dt6(self,MasProz):
        m=0.0;
        rho=0.0;
        m = MasProz / 100.0;
        rho = self.co[0];
        for n in range(1,len(self.co)):
            rho += self.co[n] * pow(m,n)
        rho = rho/1000.0;
        return rho;    
    
    #--------------------
    # Die Dichte ermittlen (Tafel 6)
    # 
    # Anders als in Tafel 6 erfordert diese Funktion
    # eine Angabe der Temperatur
    #
    # WICHTIG: die Werte müssen mit 1000
    # multipliziert werden um dem in der Tafel 6
    # gedruckten Wert zu entsprechen !
    #---------------------
    
    def Tafel6(self,VolProz,Temperatur=20.0):
        mas = 0.0
        sb = 0.0
        sc = 0.0
        ret = 0.0
      
        t1 = Temperatur - 20.0
        mas = self.Tafel4(VolProz);
        m1 = mas/100.0  
        
        for bi in range(0,len(self.cb)):
            sb += (self.cb[bi] * pow(Temperatur - 20.0, bi+1.0));
        for i in range(0,len(self.ci)):
            for k in range(0,self.ci[i]):
                sc += (self.ck[i][k] * pow(m1,k+1.0) * pow(t1, i+1.0));

        return self.dt6(mas)+ ((sb + sc) / 1000.0)

    #---------------------------
    # berechnet das tatsächliche
    # Litergewicht unter Berücksichtigung 
    # des Luftgewichts, unter Verwendung
    # der Tafel 2
    # 
    # Nicht zu verwechseln mit der Dichte 
    # aus Tafel 6 !
    # 
    #----------------------------
    def Litergewicht(self,VolProz,Temperatur):
        ret = 0.0
        if (VolProz > 0) and (VolProz <= 100):
            f3 = self.Tafel3(VolProz, Temperatur);
            ret= (1000.0 /self.Tafel2(VolProz, Temperatur)) * (f3 / 1000.0);
        else :
          ret = self.rhoWasser;
        return ret;
    
    def version(self):
        return VERSION
    
def usage():
    print(' Tafelrechner %s \nusage:\n tafelrechner.py' \
          ' --staerke=[0..99.9] --temperatur=[-20..30] --liter [zahl] --kg [zahl]' % (VERSION))
    print('tafelrechner.py -s [0..99.9] -t [-20..30] -l [zahl] -k [zahl]')
    print('Die Alkoholstärke (-s oder --staerke) MUSS angegeben werden UND >0 sein')
    
    
#------------------------
# start 
# wird ausgeführt, wenn dieses Modul als
# Programm gestartet wird,
# beim Import wird nichts weiter getan
#------------------------
if __name__ == '__main__':
    BREITE = 40
    VolProz = 0.0
    Temperatur = 20.0
    Liter = 0.0
    liter20 = Liter
    Kg =0.0
    k=False

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hs:t:l:k:", ["help", "staerke=","temperatur=","liter=","kg="])
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)
    for o, a in opts:
        if a != None:
            a = a.replace(',','.')
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        if o in("-s","--staerke"):
            VolProz = float(a) 
        if o in ("-t","--temperatur"):
            Temperatur = float(a)
        if o in ("-l","--liter"):
            Liter = float(a)
        if o in ("-k","--kg"):
            Kg = float(a)
            k = True 
    t = Tafel()
    if abs(VolProz) > 0.001 and VolProz > 0 and VolProz <100:
        print('\n' + '-' * BREITE)
        print('Alkoholstaerke\t=\t%2.2f %%Vol' %(VolProz))
        print('Tanktemperatur\t=\t%2.2f °C' %(Temperatur))
        print('Tafel 2\t\t=\t%4.1f LA in 1000 L AWM' % (t.Tafel2(VolProz, Temperatur)))
        print('Tafel 3\t\t=\t%4.1f LA in 1000 L AWM bei %2.1f °C' % (t.Tafel3(VolProz, Temperatur),Temperatur))
        print('Tafel 4\t\t=\t%4.2f %%mas' % (t.Tafel4(VolProz)))
        print('Tafel 6\t\t=\t%4.1f ' % (t.Tafel6(VolProz, Temperatur)*1000.0))  
        print("v=",VolProz)
        print('Litergewicht\t=\t%2.6f Kg/Liter' % (t.Litergewicht(VolProz, Temperatur)))
        print('Volumen\t\t=\t%4.6f Liter/Kg' % (1/t.Litergewicht(VolProz, Temperatur)))
        print('\n' + '-' * BREITE)
        if k == True:
            la = Kg * (t.Tafel2(VolProz,Temperatur)/1000.0)
        else:
            print('  %12.2f Liter bei %2.2f °C' % (Liter,Temperatur))
            la = Liter * (t.Tafel3(VolProz,Temperatur)/1000.0)
        liter20 = ((la * 100.0)/VolProz) 
        if k == False:
            Kg = liter20 * t.Litergewicht(VolProz, Temperatur)
        
        if liter20 > 0:
            print('= %12.1f Liter bei 20°C' % (liter20)) 
        print('= %12.2f Kg' % (Kg))
        print('= %12.1f LA ' % (la))
        print('\n' + '-' * BREITE)
    else:
        print("Sie müssen die Alkoholstärke angeben, die größer als 0 und kleiner als 100 sein muss.")